@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Heinz-Josef Claes (2000-2009)
+#   Copyright (C) Heinz-Josef Claes (2000-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 #
 
 
-push @VERSION, '$Id: multitail.pl 354 2009-03-15 08:39:18Z hjc $ ';
-
 sub libPath
 {
     my $file = shift;
@@ -87,39 +85,90 @@ require "tail.pl";
 require "prLog.pl";
 require 'version.pl';
 
+=head1 NAME
 
-$Help = <<EOH;
-	Read multiple log files. The log files can be written round.
-Usage:
-	$0 [-a] [-d delay] [-p begin|end] [-t]
+multitail.pl - Read multiple log files. The log files can be written round.
+
+=head1 SYNOPSIS
+
+	multitail.pl [-a] [-d delay] [-p begin|end] [-t]
                 [-o outFile [-m max] [-P]
 		 [[-n noFiles] | [-s [-c compressprog]] ]
 		] files...
 
--a --addName      add filename to the output at the beginning of each line
--d --delay        delay in sec. between checking the files (default 5 sec)
--p --position     read from begin or end of file (default = begin)
--t --withTime     with current time and date in the output
--o --out          write output to file
--m --maxFilelen   maximal len of file written round (default = 1e6)
--n --noOldFiles   number of old files to store
--P --withPID	  write pid to log file (default is not)
--H --withHostname write hostname to log file (default is not)
--l --maxlines     maximal number of lines to read per --delay in one chunk
-		  from a log file (default = 100)
-		  this value multiplied with [-d] is the number read at once
+=head1 OPTIONS
 
--s --saveLogs	  save log files with date and time instead of deleting the
-		  old (with [-noOldFiles])
--c --compressWith compress saved log files (e.g. with -c 'gzip -9')
+=over 8
 
--V		  print version(s)
+=item B<-a>, B<--addName>
 
-Copyright (c) 2001-2008 by Heinz-Josef Claes (see README)
+    add filename to the output at the beginning of each line
+
+=item B<-d>, B<--delay>
+
+    delay in sec. between checking the files (default 5 sec)
+
+=item B<-p>, B<--position>
+
+    read from begin or end of file (default = begin)
+
+=item B<-t>, B<--withTime>
+
+    with current time and date in the output
+
+=item B<-o>, B<--out>
+
+    write output to file
+
+=item B<-m>, B<--maxFilelen>
+
+    maximal len of file written round (default = 1e6)
+
+=item B<-n>, B<--noOldFiles>
+
+    number of old files to store
+
+=item B<-P>, B<--withPID>
+
+    write pid to log file (default is not)
+
+=item B<-H>, B<--withHostname>
+
+    write hostname to log file (default is not)
+
+=item B<-l>, B<--maxlines>
+
+    maximal number of lines to read per --delay in one chunk
+    from a log file (default = 100)
+    this value multiplied with [-d] is the number read at once
+
+=item B<-s>, B<--saveLogs>
+
+    save log files with date and time instead of deleting the
+    old (with [-noOldFiles])
+
+=item B<-c>, B<--compressWith>
+
+    compress saved log files (e.g. with -c 'gzip -9')
+
+=item B<-V>
+
+    print version(s)
+
+=back
+
+=head1 COPYRIGHT
+
+Copyright (c) 2001-2013 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License or any later version
-EOH
 
-&printVersions(\@ARGV, '-V');
+=cut
+
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
+
+&printVersion(\@ARGV, '-V');
 
 $CheckPar = CheckParam->new('-allowLists' => 'yes',
 			    '-list' => [
@@ -196,6 +245,7 @@ $hostname = $CheckPar->getOptWithoutPar('withHostname') ? `hostname -f` : '';
 chomp $hostname;
 $maxlines = $CheckPar->getOptWithPar('maxlines');
 $saveLogs = $CheckPar->getOptWithoutPar('saveLogs');
+$saveLogs = 'yes' if $saveLogs;
 $compressWith = $CheckPar->getOptWithPar('compressWith');
 
 unless ($CheckPar->getNoListPar())
