@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2003-2009)
+#   Copyright (C) Dr. Heinz-Josef Claes (2003-2013)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupDel.pl 354 2009-03-15 08:39:18Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 
@@ -107,11 +104,13 @@ my $keepDuplicate = '7d';
 my $checkSumFile = '.md5CheckSums';
 my $chmodMD5File = '0600';
 
-my $Help = <<EOH;
-this program deletes backups created by storeBackup
+=head1 NAME
 
-usage:
-	$prog [-f configFile] [--print]
+storeBackupDel.pl - this program deletes backups created by storeBackup
+
+=head1 SYNOPSIS
+
+	storeBackupDel.pl [-f configFile] [--print]
 	[-b backupDirectory] [-S series] [--doNotDelete]
 	[--deleteNotFinishedDirs] [-L lockFile]
 	[--keepAll timePeriod] [--keepWeekday entry] [--keepFirstOfYear]
@@ -123,25 +122,58 @@ usage:
 	 [[-n noOfOldFiles] | [--saveLogs]
 	 [--compressWith compressprog]]
 
---file		-f  configuration file (instead of parameters)
---print		    print configuration read from configuration file and stop
---backupDir	-b  top level directory of all backups (must exist)
---series	-S  directory of backup series
-		    same parameter as in storeBackup / relative path
-		    from backupDir, default is 'default'
---lockFile      -L  lock file, if exists, new instances will finish if
-		    an old is allready running, default is $lockFile
---doNotDelete	    test only, do not delete any backup
---deleteNotFinishedDirs
-		    delete old backups which where not finished
-		    this will not happen if doNotDelete is set
---keepAll	    keep backups which are not older than the specified amount
-		    of time. This is like a default value for all days in
-		    --keepWeekday. Begins deleting at the end of the script
-		    the time range has to be specified in format 'dhms', e.g.
-		       10d4h means 10 days and 4 hours
-		    default = $keepAll;
---keepWeekday	    keep backups for the specified days for the specified
+=head1 WARNING
+
+  !!! USAGE IN PARALLEL WITH storeBackup.pl CAN DESTROY YOUR BACKUPS !!!
+
+=head1 OPTIONS
+
+=over 8
+
+=item B<--file>, B<-f>
+
+    configuration file (instead of parameters)
+
+=item B<--print>
+
+    print configuration read from configuration file and stop
+
+=item B<--backupDir>, B<-b>
+
+    top level directory of all backups (must exist)
+
+=item B<--series>, B<-S>
+
+    directory of backup series
+    same parameter as in storeBackup / relative path
+    from backupDir, default is 'default'
+
+=item B<--lockFile>, B<-L>
+
+    lock file, if exists, new instances will finish if
+    an old is already running, default is $lockFile
+
+=item B<--doNotDelete>
+
+    test only, do not delete any backup
+
+=item B<--deleteNotFinishedDirs>
+
+    delete old backups which where not finished
+    this will not happen if doNotDelete is set
+
+=item B<--keepAll>
+
+    keep backups which are not older than the specified amount
+    of time. This is like a default value for all days in
+    --keepWeekday. Begins deleting at the end of the script
+    the time range has to be specified in format 'dhms', e.g.
+    10d4h means 10 days and 4 hours
+    default = $keepAll;
+
+=item B<--keepWeekday>
+
+		    keep backups for the specified days for the specified
 		    amount of time. Overwrites the default values choosen in
 		    --keepAll. 'Mon,Wed:40d Sat:60d10m' means:
 			keep backups of Mon and Wed 40days + 5mins
@@ -156,62 +188,123 @@ usage:
 			keep backups of Sat 60days + 10mins
 			keep backups of the rest of the days like specified in
 				--keepAll (default $keepAll)
---keepFirstOfYear   do not delete the first backup of a year
-		    format is timePeriod with possible 'archive flag'
---keepLastOfYear    do not delete the last backup of a year
-		    format is timePeriod with possible 'archive flag'
---keepFirstOfMonth  do not delete the first backup of a month
-		    format is timePeriod with possible 'archive flag'
---keepLastOfMonth   do not delete the last backup of a month
-		    format is timePeriod with possible 'archive flag'
---firstDayOfWeek    default: 'Sun'. This value is used for calculating
-		    --keepFirstOfWeek and --keepLastOfWeek
---keepFirstOfWeek   do not delete the first backup of a week
-		    format is timePeriod with possible 'archive flag'
---keepLastOfWeek    do not delete the last backup of a week
-		    format is timePeriod with possible 'archive flag'
---keepDuplicate     keep multiple backups of one day up to timePeriod
-		    format is timePeriod, 'archive flag' is not possible
-		    default = $keepDuplicate;
---keepMinNumber	    Keep that miminum of backups. Multiple backups of one
-		    day are counted as one backup. Default is 10.
---keepMaxNumber	    Try to keep only that maximum of backups. If you have more
-		    backups, the following sequence of deleting will happen:
-		    - delete all duplicates of a day, beginning with the old
-		      once, except the oldest of every day
-		    - if this is not enough, delete the rest of the backups
-		      beginning with the oldest, but *never* a backup with
-		      the 'archive flag' or the last backup
---keepRelative	-R  Alternative deletion scheme. If you use this option, all
-		    other keep options are ignored. Preserves backups depending
-		    on their *relative* age. Example:
-		    -R '1d 7d 2m 3m'
-		    will (try to) ensure that there is always
-		    - One backup between 1 day and 7 days old
-		    - One backup between 5 days and 2 months old
-		    - One backup between 2 months and 3 months old
-		    If there is no backup for a specified timespan
-		    (e.g. because the last backup was done more than 2 weeks
-		    ago) the next older backup will be used for this timespan.
---logFile	-l  log file (default is STDOUT)
---plusLogStdout	    if you specify a log file with --logFile you can
-		    additionally print the output to STDOUT with this flag
---suppressTime	    suppress output of time in logfile
---maxFilelen	-m  maximal length of file, default = 1e6
---noOfOldFiles	-n  number of old log files, default = 5
---saveLogs	    save log files with date and time instead of deleting the
-		    old (with [-noOldFiles])
---compressWith	    compress saved log files (e.g. with 'gzip -9')
-		    default is 'bzip2'
 
-  !!! USAGE IN PARALLEL WITH storeBackup.pl CAN DESTROY YOUR BACKUPS !!!
+=item B<--keepFirstOfYear>
 
-Copyright (c) 2003-2008 by Heinz-Josef Claes (see README)
+    do not delete the first backup of a year
+    format is timePeriod with possible 'archive flag'
+
+=item B<--keepLastOfYear>
+
+    do not delete the last backup of a year
+    format is timePeriod with possible 'archive flag'
+
+=item B<--keepFirstOfMonth>
+
+    do not delete the first backup of a month
+    format is timePeriod with possible 'archive flag'
+
+=item B<--keepLastOfMonth>
+
+    do not delete the last backup of a month
+    format is timePeriod with possible 'archive flag'
+
+=item B<--firstDayOfWeek>
+
+    default: 'Sun'. This value is used for calculating
+    --keepFirstOfWeek and --keepLastOfWeek
+
+=item B<--keepFirstOfWeek>
+
+    do not delete the first backup of a week
+    format is timePeriod with possible 'archive flag'
+
+=item B<--keepLastOfWeek>
+
+    do not delete the last backup of a week
+    format is timePeriod with possible 'archive flag'
+
+=item B<--keepDuplicate>
+
+    keep multiple backups of one day up to timePeriod
+    format is timePeriod, 'archive flag' is not possible
+    default = $keepDuplicate;
+
+=item B<--keepMinNumber>
+
+    Keep that miminum of backups. Multiple backups of one
+    day are counted as one backup. Default is 10.
+
+=item B<--keepMaxNumber>
+
+    Try to keep only that maximum of backups. If you have
+    more backups, the following sequence of deleting will
+    happen:
+	    - delete all duplicates of a day, beginning with the
+              old once, except the oldest of every day
+	    - if this is not enough, delete the rest of the backups
+	      beginning with the oldest, but *never* a backup with
+	      the 'archive flag' or the last backup
+
+=item B<--keepRelative>, B<-R>
+
+    Alternative deletion scheme. If you use this option, all other
+    keep options are ignored. Preserves backups depending
+    on their *relative* age. Example:
+    -R '1d 7d 2m 3m'
+        will (try to) ensure that there is always
+	- One backup between 1 day and 7 days old
+	- One backup between 5 days and 2 months old
+	- One backup between 2 months and 3 months old
+	If there is no backup for a specified timespan
+	(e.g. because the last backup was done more than 2 weeks
+	ago) the next older backup will be used for this timespan.
+
+=item B<--logFile>, B<-l>
+
+    log file (default is STDOUT)
+
+=item B<--plusLogStdout>
+
+    if you specify a log file with --logFile you can
+    additionally print the output to STDOUT with this flag
+
+=item B<--suppressTime>
+
+    suppress output of time in logfile
+
+=item B<--maxFilelen>, B<-m>
+
+    maximal length of file, default = 1e6
+
+=item B<--noOfOldFiles>, B<-n>
+
+    number of old log files, default = 5
+
+=item B<--saveLogs>
+
+    save log files with date and time instead of deleting the
+    old (with [-noOldFiles])
+
+=item B<--compressWith>
+
+    compress saved log files (e.g. with 'gzip -9')
+    default is 'bzip2'
+
+=back
+
+=head1 COPYRIGHT
+
+Copyright (c) 2003-2013 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
-EOH
-    ;
 
-&printVersions(\@ARGV, '-V');
+=cut
+
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
+
+&printVersion(\@ARGV, '-V');
 
 my $startDate = dateTools->new();
 
@@ -350,7 +443,12 @@ my $CheckPar =
 					    '-cl_option' => '--today',
 					    '-cf_key' => 'today',
 					    '-hidden' => 'yes',
-					    '-param' => 'yes')
+					    '-param' => 'yes'),
+# used by storeBackupMount.pl
+				Option->new('-name' => 'writeToNamedPipe',
+					    '-cl_option' => '--writeToNamedPipe',
+					    '-param' => 'yes',
+					    '-hidden' => 'yes')
 				]
 		    );
 
@@ -391,10 +489,11 @@ my $maxFilelen = $CheckPar->getOptWithPar('maxFilelen');
 my $noOfOldFiles = $CheckPar->getOptWithPar('noOfOldFiles');
 my $saveLogs = $CheckPar->getOptWithoutPar('saveLogs') ? 'yes' : 'no';
 my $compressWith = $CheckPar->getOptWithPar('compressWith');
-
+# hidden options
 my $printAll = $CheckPar->getOptWithoutPar('printAll');
 $print = 1 if $printAll;
 my $todayOpt = $CheckPar->getOptWithPar('todayOpt');  # format like
+my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
                                                       # backup dir name
 
 if ($print)
@@ -403,29 +502,51 @@ if ($print)
     exit 0;
 }
 
-my $prLog;
-my (@kind) = ('I:INFO', 'W:WARNING', 'E:ERROR', 'S:STATISTIC',
-	      'D:DEBUG', 'V:VERSION');
+my $prLog1;
+my (@kind) = ('A:BEGIN', 'Z:END','I:INFO', 'W:WARNING', 'E:ERROR',
+	      'S:STATISTIC', 'D:DEBUG', 'V:VERSION');
 
 if ($logFile)
 {
-    $prLog = printLog->new('-kind' => \@kind,
-			   '-file' => $logFile,
-			  '-withTime' => $withTime,
-			  '-maxFilelen' => $maxFilelen,
-			  '-noOfOldFiles' => $noOfOldFiles);
+    $prLog1 = printLog->new('-kind' => \@kind,
+			    '-file' => $logFile,
+			    '-withTime' => $withTime,
+			    '-maxFilelen' => $maxFilelen,
+			    '-noOfOldFiles' => $noOfOldFiles);
 }
 else
 {
-    $prLog = printLog->new('-kind' => \@kind);
+    $prLog1 = printLog->new('-kind' => \@kind);
 }
 
+my $prLog = printLogMultiple->new('-prLogs' => [$prLog1]);
+
+if ($plusLogStdout)
+{
+    my $p = printLog->new('-kind' => \@kind,,
+			  '-filedescriptor', *STDOUT);
+    $prLog->add('-prLogs' => [$p]);
+}
+if ($writeToNamedPipe)
+{
+    my $np = printLog->new('-kind' => \@kind,
+			   '-file' => $writeToNamedPipe,
+			   '-maxFilelen' => 0);
+    $prLog->add('-prLogs' => [$np]);
+}
+
+(@main::cleanup) = ($prLog, undef);
+$SIG{INT} = \&cleanup;
+$SIG{TERM} = \&cleanup;
+
+$prLog->print('-kind' => 'A',
+	      '-str' =>
+	      ["starting deletion in <$backupDir>, series <$series>"]);
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"]);
+	      '-str' => ["storeBackupDel.pl, $main::STOREBACKUPVERSION"]);
 
 $prLog->print('-kind' => 'E',
-	      '-str' => ["backupDir directory <$backupDir> does not exist\n$Help\n"],
+	      '-str' => ["backupDir directory <$backupDir> does not exist\n$Help"],
 	      '-exit' => 1)
     unless -e $backupDir;
 
@@ -459,14 +580,14 @@ if ($lockFile)
 	close(FILE);
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["strange format in lock file <$lockFile>, " .
-				 "line is <$pid>\n"],
+				 "line is <$pid>"],
 		      '-exit' => 1)
 	    unless ($pid =~ /\A\d+\Z/o);
 	if (kill(0, $pid) == 1)   # alte Instanz luft noch
 	{
 	    $prLog->print('-kind' => 'E',
 			  '-str' => ["cannot start, old instance with pid " .
-				     "<$pid> is allready running"],
+				     "<$pid> is already running"],
 			  '-exit' => 1);
 	}
 	else
@@ -581,4 +702,20 @@ $prLog->print('-kind' => 'I',
 	      '-str' => ["removing lock file <$lockFile>"]);
 unlink $lockFile;
 
+$prLog->print('-kind' => 'Z',
+	      '-str' =>
+	      ["finished deletion in <$backupDir>, series <$series>"]);
+
+
 exit 0;
+
+
+##################################################
+# package printLogMultiple needs this function
+sub cleanup
+{
+    my $signame = shift;
+    my $exit = (shift() || 1);
+
+    exit $exit;
+}
