@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002-2009)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupRecover.pl 354 2009-03-15 08:39:18Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 
@@ -106,38 +103,84 @@ my $checkSumFile = '.md5CheckSums';
 my $tmpdir = '/tmp';              # default value
 $tmpdir = $ENV{'TMPDIR'} if defined $ENV{'TMPDIR'};
 
+=head1 NAME
 
-my $Help = <<EOH;
-This program recovers files saved with storeBackup.pl.
+storeBackupRecover.pl - recovers files saved with storeBackup.pl.
 
-usage:
-	$prog -r restore [-b root] -t targetDir [--flat]
+=head1 SYNOPSIS
+
+	storeBackupRecover.pl -r restore [-b root] -t targetDir [--flat]
 		[-o] [--tmpdir] [--noHardLinks] [-p number] [-v] [-n]
 		[--cpIsGnu] [--noGnuCp]
 
---restoreTree	    -r  file or (part of) the tree to restore
-		        when restoring a file, the file name in the backup has
-		        to be used (eg. with compression suffix)
---backupRoot	    -b  root of storeBackup tree, normally not needed
---targetDir	    -t  directory for unpacking
---flat                  do not create subdirectories
---overwrite	    -o  overwrite existing files
---tmpdir	    -T  directory for temporary file, default is <$tmpdir>
---noHardLinks		do not reconstruct hard links in restore tree
---noRestoreParallel -p  max no of paralell programs to unpack, default is $noRestoreParallel
---verbose	    -v  print verbose messages
---noRestored	    -n  print number of restored dirs, hardlinks, symlinks, files, ...
---noGnuCp	        overwrite information in backup: you do not have gnucp
-			installed
-			(only relevant for sockets, block and character devices)
+=head1 OPTIONS
 
-Copyright (c) 2002-2008 by Heinz-Josef Claes (see README)
+=over 8
+
+=item B<--restoreTree>, B<-r>
+
+    file or (part of) the tree to restore
+    when restoring a file, the file name in the backup has
+    to be used (eg. with compression suffix)
+
+=item B<--backupRoot>, B<-b>
+
+    root of storeBackup tree, normally not needed
+
+=item B<--targetDir>, B<-t>
+
+    directory for unpacking
+
+=item B<--flat>
+
+    do not create subdirectories
+
+=item B<--overwrite>, B<-o>
+
+    overwrite existing files
+
+=item B<--tmpdir>, B<-T>
+
+    directory for temporary file, default is <$tmpdir>
+
+=item B<--noHardLinks>
+
+    do not reconstruct hard links in restore tree
+
+=item B<--noRestoreParallel>, B<-p>
+
+    max no of paralell programs to unpack, default is 12
+    reduce this number if you are restoring blocked files
+    and the system has insufficient RAM
+
+=item B<--verbose>, B<-v>
+
+    print verbose messages
+
+=item B<--noRestored>, B<-n>
+
+    print number of restored dirs, hardlinks, symlinks, files, ...
+
+=item B<--noGnuCp>
+
+    overwrite information in backup: you do not have gnucp
+    installed
+    (only relevant for sockets, block and character devices)
+
+=back
+
+=head1 COPYRIGHT
+
+Copyright (c) 2002-2013 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
-EOH
-    ;
 
+=cut
 
-&printVersions(\@ARGV, '-V');
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
+
+&printVersion(\@ARGV, '-V');
 
 my $CheckPar =
     CheckParam->new('-allowLists' => 'no',
@@ -202,6 +245,7 @@ my $noGnuCp = $CheckPar->getOptWithoutPar('noGnuCp');
 
 my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
 				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION']);
+$prLog->fork($req);
 
 $prLog->print('-kind' => 'E',
 	      '-str' => ["target directory <$targetDir> does not exist"],
@@ -209,8 +253,7 @@ $prLog->print('-kind' => 'E',
     unless (-d $targetDir);
 
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"])
+	      '-str' => ["storeBackupRecover.pl, $main::STOREBACKUPVERSION"])
     if $verbose;
 
 my $rt = $restoreTree;
@@ -265,12 +308,11 @@ else
 
 
     $prLog->print('-kind' => 'E',
-		  '-str' => ["did not find info file <$checkSumFile>\n"],
+		  '-str' => ["did not find info file <$checkSumFile>"],
 		  '-exit' => 1)
 	unless ($backupRoot);
 }
 
-#$restoreTree =~ s/$backupRoot\/*//;
 $restoreTree = substr($restoreTree, length($backupRoot) + 1);
 
 
@@ -284,11 +326,18 @@ $prLog->print('-kind' => 'E',
 	      '-exit' => 1)
     if -e "$backupRoot/.storeBackupLinks/linkFile.bz2";
 
-
+my (%setPermDirs);
 unless ($flat)
 {
     # Subtree unter dem Zieldirectory erzeugen
     &::makeFilePath("$targetDir/$restoreTree", $prLog);
+
+    my (@d) = split(/\/+/, $restoreTree);
+    my $i;
+    for ($i = 0 ; $i < @d ; $i++)
+    {
+	$setPermDirs{join('/', @d[0..$i])} = 1;
+    }
 }
 
 #
@@ -302,12 +351,16 @@ my $rcsf = readCheckSumFile->new('-checkSumFile' =>
 my $fork = parallelFork->new('-maxParallel' => $noRestoreParallel,
 			     '-prLog' => $prLog);
 
-my $meta = $rcsf->getMetaValField();
+#my $meta = $rcsf->getMetaValField();
 
-my ($uncompr, @uncomprPar) = @{$$meta{'uncompress'}};
+#my ($uncompr, @uncomprPar) = @{$$meta{'uncompress'}};
+my ($uncompr, @uncomprPar) = @{$rcsf->getInfoWithPar('uncompress')};
+#my ($cp, @cpPar) = ('cp', '-dPR');
 my ($cp, @cpPar) = ('cp', '-dPR');
-my $postfix = ($$meta{'postfix'})->[0];
-my $gnucp = ($$meta{'cpIsGnu'})->[0];
+#my $postfix = ($$meta{'postfix'})->[0];
+my $postfix = $rcsf->getInfoWithPar('postfix');
+#my $gnucp = ($$meta{'cpIsGnu'})->[0];
+my $gnucp = $rcsf->getInfoWithPar('cpIsGnu');
 $gnucp = ($gnucp eq 'yes') ? 1 : 0;
 $gnucp = 0 if $noGnuCp;
 
@@ -347,10 +400,19 @@ my $noCharDevs = 0;
 my $noDirs = 0;
 my $hardLinks = 0;
 
-# Zurcksichern der Dateien
-$restoreTree =~ s/\/\.//g;
 $restoreTree = '' if $restoreTree eq '.';
 my $lrestoreTree = length($restoreTree);
+
+my $tmpDirFile = &::uniqFileName("$tmpdir/stbuRec.");
+&::checkDelSymLink($tmpDirFile, $prLog, 0x01);
+local *DIRFILE;
+open(DIRFILE, '>', $tmpDirFile) or
+    $prLog->print('-kind' => 'E',
+		  '-str' => ["cannot open <$tmpDirFile>, exiting"],
+		  '-add' => [__FILE__, __LINE__],
+		  '-exit' => 1);
+chmod 0600, $tmpDirFile;
+
 my ($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
     $size, $uid, $gid, $mode, $filename);
 #print "restoreTree = <$restoreTree>\n";
@@ -360,13 +422,19 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	 $size, $uid, $gid, $mode, $filename) = $rcsf->nextLine()) > 0)
 {
     my $f = $filename;
+    if (exists($setPermDirs{$f}))
+    {
+	chown $uid, $gid, "$targetDir/$f";
+	chmod $mode, "$targetDir/$f";
+	utime $atime, $mtime, "$targetDir/$f";
+    }
     if ($isFile and length($md5sum) == 32)
-#	($md5sum ne 'dir' or $md5sum ne 'symlink' or $md5sum ne 'pipe'))
     {
 	$f .= $postfix if ($compr eq 'c');
     }
 #print "from .md5CheckSums: <$f> <$restoreTree> $lrestoreTree\n";
-    if ($restoreTree eq '' or "$restoreTree/" eq substr("$f/", 0, $lrestoreTree + 1)
+    if ($restoreTree eq ''
+	or "$restoreTree/" eq substr("$f/", 0, $lrestoreTree + 1)
 	or ($isFile and $restoreTree eq $f))
     {
 #print "---> restore!\n";
@@ -396,8 +464,13 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 			      '-exit' => 1)
 		    unless mkdir $targetFile;
 		chown $uid, $gid, $targetFile;
-		chmod $mode, $targetFile;
-		utime $atime, $mtime, $targetFile;
+#		chmod $mode, $targetFile;
+#		utime $atime, $mtime, $targetFile;
+
+		my $wr = $targetFile;
+		$wr =~ s/\n/\0/og;
+		print DIRFILE "$atime $mtime $mode $wr\n";
+
 		$prLog->print('-kind' => 'I',
 			      '-str' => ["mkdir $targetFile"])
 		    if ($verbose);
@@ -628,7 +701,6 @@ contLoop:;
 }
 
 finish:;
-close(FILE);
 
 my $job;
 while ($job = $fork->waitForAllJobs())
@@ -651,6 +723,34 @@ unless ($noHardLinks)
     unlink "$tmpdir/stbrecover.$$";
 }
 
+# set atime, mtime, mode of directories
+close(DIRFILE);
+unless (open(DIRFILE, '<', $tmpDirFile))
+{
+    $prLog->print('-kind' => 'E',
+		  '-str' => ["cannot read <$tmpDirFile>, cannot set " .
+			     "atime and mtime for directories"]);
+}
+else
+{
+    $prLog->print('-kind' => 'I',
+		  '-str' => ["setting atime, mtime of directories ..."])
+	if $verbose;
+
+    my $line;
+    while ($line = <DIRFILE>)
+    {
+	chop $line;
+	my ($atime, $mtime, $mode, $df) = split(/\s/, $line, 4);
+	$df =~ s/\0/\n/og;
+	chmod $mode, $df;
+	utime $atime, $mtime, $df;
+	utime $atime, $mtime, $df;
+    }
+    close(DIRFILE);
+}
+unlink $tmpDirFile;
+
 $prLog->print('-kind' => 'I',
 	      '-str' =>
 	      [join(', ',
@@ -746,24 +846,31 @@ sub uncompressCatBlock
 
     foreach $entry (sort @entries)
     {
+	my $buffer;
 	local *FROM;
+	my $fileIn = undef;
 	if ($entry =~ /$postfix\Z/)    # compressed block
 	{
 	    if ($main::IOCompressDirect)
 	    {
 		my $input = "$fromDir/$entry";
-		my $buffer;
 		my $uc = new IO::Uncompress::Bunzip2 $input;
-		$uc->read($buffer);
-		syswrite(TO, $buffer);
+		while ($uc->read($buffer, 10*1024**2))
+		{
+		    syswrite(TO, $buffer);
+		}
 		next;
 	    }
 
-	    unless (open(FROM, "$uncompr @$uncomprPar < \'$fromDir/$entry\' |"))
+	    $fileIn =
+		pipeFromFork->new('-exec' => $uncompr,
+				  '-param' => \@uncomprPar,
+				  '-stdin' => "$fromDir/$entry",
+				  '-outRandom' => '/tmp/stbuPipeFrom11-',
+				  '-prLog' => $prLog);
+	    while ($fileIn->sysread(\$buffer, 10*1024**2))
 	    {
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["cannot read <$fromDir/$entry>"]);
-		return 1;
+		syswrite(TO, $buffer);
 	    }
 	}
 	else           # block not compressed
@@ -774,13 +881,21 @@ sub uncompressCatBlock
 			      '-str' => ["cannot read <$fromDir/$entry>"]);
 		return 1;
 	    }
+	    while (sysread(FROM, $buffer, 10*1024**2))
+	    {
+		syswrite(TO, $buffer);
+	    }
 	}
-	my $buffer;
-	while (sysread(FROM, $buffer, 10*1024**2))
+
+	if ($fileIn)
 	{
-	    syswrite(TO, $buffer);
+	    $fileIn->close();
+	    $fileIn = undef;
+	}
+	else
+	{
+	    close(FROM);
 	}
-	close(FROM);
     }
     close(TO);
     return 0;
