@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008-2009)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupSearch.pl 341 2008-07-31 13:13:17Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 
@@ -102,44 +99,89 @@ require 'storeBackupLib.pl';
 
 my $checkSumFile = '.md5CheckSums';
 
-my $Help = <<EOH;
-This program locates different versions of a file saved with storeBackup.pl.
-You need some basic understanding of linux and perl to use it.
+=head1 NAME
 
-usage:
-	$prog -g configFile
+storeBackupSearch.pl - locates different versions of a file saved with storeBackup.pl.
 
-	$prog [-f configFile] [-b backupDirDir]
+=head1 SYNOPSIS
+
+	storeBackupSearch.pl -g configFile
+
+	storeBackupSearch.pl -b backupDirDir [-f configFile]
 	      [-s rule]  [--absPath] [-w file] [--parJobs number]
 	      [-d level] [--once] [--print] [backupRoot . . .]
 
---generate	-g  generate a config file
---print		    print configuration read from configuration file and stop
---configFile	-f  configuration file (instead of or
-		    additionally to parameters)
---backupDir	-b  *** param must exist ***
+=head1 DESCRIPTION
+
+You need some basic understanding of linux and perl to use it.
+
+=head1 OPTIONS
+
+=over 8
+
+=item B<--generate>, B<-g>
+
+    generate a config file
+
+=item B<--print>
+
+    print configuration read from configuration file and stop
+
+=item B<--configFile>, B<-f>
+
+    configuration file (instead of or
+    additionally to parameters)
+
+=item B<--backupDir> B<-b>
+
 		    top level directory of all backups
---searchRule	-s  *** param must exist ***
+
+=item B<--searchRule>, B<-s>
+
 		    rule for searching
 		    see README: 'including / excluding files and directories'
---absPath	-a  write result with absolute path names
---writeToFile	-w  write search result also to file
---parJobs	-p  number of parallel jobs, default = choosen automaticly
---debug		-d  debug level, possible values are 0, 1, 2, default = 0
---once		-o  show every file found only once (depending on md5 sum)
-backupRoot	    Root directories of backups where to search relative
-		    to backupDir. If no directories are specified, all
-		    backups below backupDir are choosen.
 
-Copyright (c) 2008 by Heinz-Josef Claes (see README)
+=item B<--absPath>, B<-a>
+
+    write result with absolute path names
+
+=item B<--writeToFile>, B<-w>
+
+    write search result also to file
+
+=item B<--parJobs>, B<-p>
+
+    number of parallel jobs, default = choosen automaticly
+
+=item B<--debug>, B<-d>
+
+    debug level, possible values are 0, 1, 2, default = 0
+
+=item B<--once>, B<-o>
+
+    show every file found only once (depending on md5 sum)
+
+=item backupRoot
+
+    Root directories of backups where to search relative
+    to backupDir. If no directories are specified, all
+    backups below backupDir are choosen.
+
+=back
+
+=head1 COPYRIGHT
+
+Copyright (c) 2008-2013 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
-EOH
-    ;
 
-# '
+=cut
+
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
 
 my $templateConfigFile = <<EOC;
-# configuration file for storeBackup.pl
+# configuration file for storeBackupSearch.pl, version $main::STOREBACKUPVERSION
 
 # You can set a value specified with '-cf_key' (eg. logFiles) and
 # continue at the next lines which have to begin with a white space:
@@ -182,7 +224,7 @@ my $templateConfigFile = <<EOC;
 
 # write result with absolute path names
 # default is 'no', possible values are 'yes' and 'no'
-;absPath
+;absPath=
 
 # write search result also to file
 ;writeToFile=
@@ -200,7 +242,7 @@ EOC
     ;
 
 
-&printVersions(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V');
 
 my $CheckPar =
     CheckParam->new('-configFile' => '-f',
@@ -340,10 +382,10 @@ if ($print)
 
 my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
 				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION']);
+$prLog->fork($req);
 
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"]);
+	      '-str' => ["storeBackupSearch.pl, $main::STOREBACKUPVERSION"]);
 
 $prLog->print('-kind' => 'E',
 	      '-str' => ["missing parameters backupDir and searchRule\n$Help"],
@@ -484,8 +526,9 @@ foreach $dirToSearch (@dirsToSearch)
     my $rcsf =
 	readCheckSumFile->new('-checkSumFile' => "$dirToSearch/$checkSumFile",
 			      '-prLog' => $prLog);
-    my $meta = $rcsf->getMetaValField();
-    my $postfix = ($$meta{'postfix'})->[0];    # postfix for compression
+#    my $meta = $rcsf->getMetaValField();
+#    my $postfix = ($$meta{'postfix'})->[0];    # postfix for compression
+    my $postfix = $rcsf->getInfoWithPar('postfix');
 
     my $jobToDo = 1;
     my $parForkToDo = 1;
