@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002-2009)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupVersions.pl 354 2009-03-15 08:39:18Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 use Fcntl qw(O_RDWR O_CREAT);
@@ -103,40 +100,87 @@ require 'humanRead.pl';
 require 'dateTools.pl';
 require 'storeBackupLib.pl';
 
-
 my $checkSumFile = '.md5CheckSums';
 
-my $Help = <<EOH;
-This program locates different versions of a file saved with storeBackup.pl.
+=head1 NAME
 
-usage:
-	$prog -f file [-b root]  [-v]
+storeBackupVersions.pl - locates different versions of a file saved with storeBackup.pl.
+
+=head1 SYNOPSIS
+
+	storeBackupVersions.pl -f file [-b root]  [-v]
 	 [-l [-a | [-s] [-u] [-g] [-M] [-c] [-m]]]
 
---file		-f  file name (name in the backup, probably with suffix
-		    from compression)
---backupRoot	-b  root of storeBackup tree, normally not needed
---verbose	-v  print verbose messages
---locateSame	-l  locate same file with other names
---showAll	-A  same as: [-s -u -g -M -c -m]
---size		-s  show size (human readable) of source file
---uid		-u  show uid of source file
---gid		-g  show gid of source file
---mode		-M  show permissions of source file
---ctime		-c  show creation time of source file
---mtime		-m  show modify time of source file
---atime		-a  show access time of source file
+=head1 OPTIONS
+
+=over 8
+
+=item B<--file>, B<-f>
+
+    file name (name in the backup, probably with suffix
+    from compression)
+
+=item B<--backupRoot> B<-b>
+
+    root of storeBackup tree, normally not needed
+
+=item B<--verbose>, B<-v>
+
+    print verbose messages
+
+=item B<--locateSame>, B<-l>
+
+    locate same file with other names
+
+=item B<--showAll>, B<-A>
+
+    same as: [-s -u -g -M -c -m]
+
+=item B<--size>, B<-s>
+
+    show size (human readable) of source file
+
+=item B<--uid>, B<-u>
+
+    show uid of source file
+
+=item B<--gid>, B<-g>
+
+    show gid of source file
+
+=item B<--mode>, B<-M>
+
+    show permissions of source file
+
+=item B<--ctime>, B<-c>
+
+    show creation time of source file
+
+=item B<--mtime>, B<-m>
+
+    show modify time of source file
+
+=item B<--atime>, B<-a>
+
+    show access time of source file
+
+=back
 
 It does not always work correctly when a file is saved blocked *and*
-non-blocked in different backups. In such cases, use option --locateSame.
+non-blocked in different backups. In such cases, use option B<--locateSame>.
 
-Copyright (c) 2002-2008 by Heinz-Josef Claes (see README)
+=head1 COPYRIGHT
+
+Copyright (c) 2002-2013 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
-EOH
-    ;
 
+=cut
 
-&printVersions(\@ARGV, '-V');
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
+
+&printVersion(\@ARGV, '-V');
 
 my $CheckPar =
     CheckParam->new('-allowLists' => 'no',
@@ -224,8 +268,7 @@ my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
 				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION']);
 
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"])
+	      '-str' => ["storeBackupVersions.pl, $main::STOREBACKUPVERSION"])
     if $verbose;
 
 #
@@ -270,7 +313,7 @@ else
     } while ($dir ne '/');
 
     $prLog->print('-kind' => 'E',
-		  '-str' => ["did not find info file <$checkSumFile>\n"],
+		  '-str' => ["did not find info file <$checkSumFile>"],
 		  '-exit' => 1)
 	unless ($backupRoot);
 }
@@ -452,8 +495,9 @@ foreach $entry (@versionDirs)   # jetzt alle zur Datei *gespeicherten*
     my $rcsf = readCheckSumFile->new('-checkSumFile' => "$entry/$checkSumFile",
 				     '-prLog' => $prLog);
 				     
-    my $meta = $rcsf->getMetaValField();
-    my $postfix = ($$meta{'postfix'})->[0];    # postfix (kompr. oder nicht) merken
+#    my $meta = $rcsf->getMetaValField();
+#    my $postfix = ($$meta{'postfix'})->[0];    # postfix (kompr. oder nicht) merken
+    my $postfix = $rcsf->getInfoWithPar('postfix');
 
     my ($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	$size, $uid, $gid, $mode, $filename);
