@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2002-2008)
+#   Copyright (C) Heinz-Josef Claes (2002-2012)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -48,9 +48,14 @@
 #$CheckPar->check('-argv' => \@ARGV,
 #                 '-help' => $Help,
 #                 );
+#my (@optNames) = $CheckPar->getOptNamesSet('-type' => 'withPar');
+#my (@optNames) = $CheckPar->getOptNamesSet('-type' => 'withoutPar');
 #my $configFile = $CheckPar->getOptWithPar('confFile');
 #my $verbose = $CheckPar->getOptWithoutPar('verbose');
 #$CheckPar->print();
+#my (@list) = $CheckPar->getListPar();
+#my numberListPar = $CheckPar->getNoListPar();
+#my (@list) = $CheckPar->getOptOrder();
 #
 ######################################################################
 # Options of Option::new
@@ -146,16 +151,22 @@
 #                Use this only if you use a configuration file.
 #                The option pointed to must only have '-cf_key', not
 #                '-cl_option' or '-cl_alias'.
+# '-replaceEnvVar' 'yes' (default) or 'no'. Default is to replace
+#                environment variables (beginning with $, eg. $PWD)
+# '-ignoreLeadingWhiteSpace' If set to 1, ignore leading white spaces
+#                for options in configuration file. Default is undef.
+#                -> this means, that no continuation of a line in the
+#                -> next line is possible!
 #
 ######################################################################
 # Options of CheckParam::check
 #
 # '-argv'        Pointer to argument vector of skript, eg. \@ARGV
 # '-help'        Help text which will be written in case of an error
-# '-ignoreAdditionalKeys' If set, do not generate an error message
+# '-ignoreAdditionalKeys' If set to 1, do not generate an error message
 #                when keys are found, which are not described with
 #                Option objects. Useful when reading configuration
-#                files only partially.
+#                files only partially. Default is undef.
 #
 ######################################################################
 # Options of CheckParam::getOptWithPar
@@ -217,8 +228,6 @@
 
 
 
-push @VERSION, '$Id: checkParam.pl,v 1.12 2002/05/20 06:45:34 hjc Exp $ ';
-
 use strict;
 
 require 'checkObjPar.pl';
@@ -349,7 +358,9 @@ sub new
     my ($self) = {};
 
     # Defaultwerte f"ur Parameter setzen
-    my (%params) = ('-file' => undef
+    my (%params) = ('-file' => undef,
+		    '-replaceEnvVar' => 'yes',
+		    '-ignoreLeadingWhiteSpace' => undef
 	);
 
     &::checkObjectParams(\%params, \@_, 'ConfigFile::new',
@@ -357,10 +368,27 @@ sub new
     &::setParamsDirect($self, \%params);
     bless $self, $class;
 
+    die "replaceEnvVar must be 'yes' or 'no' in ConfigFile::new"
+	unless $self->{'replaceEnvVar'} =~ /\Ayes\Z|\Ano\Z/;
+    my $ignoreLeadingWhiteSpace = $self->{'ignoreLeadingWhiteSpace'};
+
     # read config file
     my $file = $self->{'file'};
     local *FILE;
-    open(FILE, $file) or die "cannot open configuration file <$file>";
+    if ($file =~ /\.bz2\Z/)
+    {
+	open(FILE, "bzip2 -d < \'$file\' |") or
+	    die "cannot open configuration file <$file>";
+    }
+    elsif ($file =~ /\.gz\Z/)
+    {
+	open(FILE, "gzip -d < \'$file\' |") or
+	    die "cannot open configuration file <$file>";
+    }
+    else
+    {
+	open(FILE, $file) or die "cannot open configuration file <$file>";
+    }
     my (@configFile) = <FILE>;
     chomp @configFile;
     close(FILE);
@@ -374,7 +402,10 @@ sub new
     foreach $l (@cf)
     {
 	$i++;
-	if ($l =~ /\A#/ or $l =~ /\A\s*\Z/)  # comment or empty
+	$l =~ s/\A\s+(.*?)\Z/$1/
+	    if $ignoreLeadingWhiteSpace;
+
+	if ($l =~ /\A[#;]/ or $l =~ /\A\s*\Z/)  # comment or empty
 	{
 	    $actKey = undef;
 	    next;
@@ -407,9 +438,17 @@ sub new
     foreach $actKey (keys %keys)
     {
 	$l = $keys{$actKey}{'line'};
-	$keys{$actKey}{'parts'} =
-	    splitQuotedLine($l, "in configuration file at line " .
-			    $keys{$actKey}{'lineno'});
+	my $v = splitQuotedLine($l, "in configuration file at line " .
+				$keys{$actKey}{'lineno'},
+				$self->{'replaceEnvVar'} eq 'no');
+	if (@$v)
+	{
+	    $keys{$actKey}{'parts'} = $v;
+	}
+	else
+	{
+	    delete $keys{$actKey};   # value is not set in config file
+	}
     }
 
     return $self;
@@ -420,6 +459,9 @@ sub splitQuotedLine
 {
     my $l = shift;
     my $errorPart = shift;   # where this happens, 'in file at ...'
+    my $doNotReplaceEnvVar = shift;   # 1 or undef
+
+    $doNotReplaceEnvVar = 0 unless defined $doNotReplaceEnvVar;
 
     # masking of special characters
     my $dollar = "\001";  # mask for \$
@@ -432,11 +474,10 @@ sub splitQuotedLine
     $l =~ s/\\\{/$ob/g;    # replace \{ with $ob
     $l =~ s/\\\}/$cb/g;    # replace \} with $cb
     $l =~ s/\\\'/$sq/g;    # replace \' with $sq
-    $l =~ s/\\\"/$dq/g;    # replace \" with $sq
+    $l =~ s/\\\"/$dq/g;    # replace \" with $dq
 
     my (@l);
-
-#print "==> $actKey\n\t<$l>\n";
+#print "l = <$l>, doNotReplaceEnvVar = <$doNotReplaceEnvVar>\n";
     while (length($l))
     {
 	$l =~ s/\A\s*(.*?)\s*\Z/$1/;   # remove leading and trailing \s
@@ -446,7 +487,9 @@ sub splitQuotedLine
 #print "dQuote = $dQuote\n";
 	if ($sQuote == -1 and $dQuote == -1)   # no quotes
 	{
-	    push @l, replaceEnvironmentVars(split(/\s+/, $l));
+	    push @l, replaceEnvironmentVars($doNotReplaceEnvVar,
+					    $errorPart,
+					    split(/\s+/, $l));
 	    $l = '';
 	}
 	elsif ($dQuote == -1 or
@@ -455,7 +498,9 @@ sub splitQuotedLine
 #print "-1- <$l>\n";
 	    if ($l =~ /\A(.*?)\'(.*?)\'(.*)\Z/)
 	    {
-		push @l, replaceEnvironmentVars(split(/\s+/, $1));
+		push @l, replaceEnvironmentVars($doNotReplaceEnvVar,
+						$errorPart,
+						split(/\s+/, $1));
 		push @l, $2;
 		$l = $3;
 #print "\t<", join('> <', @l), "> + <$l>\n";
@@ -470,8 +515,11 @@ sub splitQuotedLine
 #print "-2- <$l>\n";
 	    if ($l =~ /\A(.*?)\"(.*?)\"(.*)\Z/)
 	    {
-		push @l, replaceEnvironmentVars(split(/\s+/, $1));
-		push @l, replaceEnvironmentVars($2);
+		push @l, replaceEnvironmentVars($doNotReplaceEnvVar,
+						$errorPart,
+						split(/\s+/, $1));
+		push @l, replaceEnvironmentVars($doNotReplaceEnvVar,
+						$errorPart, $2);
 #print "\t<", join('> <', @l), "> + <$l>\n";
 		$l = $3;
 	    }
@@ -492,6 +540,7 @@ sub splitQuotedLine
 	$l[$i] =~ s/$dq/\"/g;
     }
 
+#print "-3- \@l = <", join('><', @l), ">\n";
     return \@l;
 }
 
@@ -499,7 +548,9 @@ sub splitQuotedLine
 ########################################
 sub replaceEnvironmentVars
 {
-    my (@lines) = @_;
+    my ($doNotReplaceEnvVar, $errorPart, @lines) = @_;
+
+    return (@lines) if $doNotReplaceEnvVar;
 
     my (@newLines);
     my $l;
@@ -527,7 +578,7 @@ sub replaceEnvironmentVars
 		}
 
 		die "environment variable \$$env not set\n",
-		"please set \$$env before calling this program"
+		"please set \$$env before calling this program $errorPart"
 		    unless $ENV{$env};
 
 		$l =~ s/\$\{?$env\}?/$ENV{$env}/;
@@ -560,7 +611,9 @@ sub new
     my (%params) = ('-list' => [],
 		    '-configFile' => undef,
 		    '-allowLists' => 'no',
-		    '-listMapping' => undef);
+		    '-listMapping' => undef,
+		    '-replaceEnvVar' => 'yes',
+		    '-ignoreLeadingWhiteSpace' => undef);
 
     $params{'-allowLists'} = 'yes'
 	if defined $params{'-listMapping'};
@@ -624,6 +677,8 @@ sub check
 		    '-help' => ['<noHelp>'],
 		    '-ignoreAdditionalKeys' => undef);
 
+    my $replaceEnvVar =
+	$self->{'paramList'}{'-replaceEnvVar'};
     my (%hash) = @_;    # Parameter in Hash kopieren
     my ($k);
     foreach $k (keys %params)
@@ -654,12 +709,13 @@ sub check
     }
 
     my ($arg, @parList, $next, $aktOpt, %optWithPar,
-	%optWithoutPar, %unset, $i);
+	@optOrder, %optWithoutPar, %unset, $i);
 
     # Ergebnisse merken
     $self->{'Erg'}{'withPar'} = \%optWithPar;
     $self->{'Erg'}{'withoutPar'} = \%optWithoutPar;
     $self->{'Erg'}{'listPar'} = \@parList;
+    $self->{'Erg'}{'optOrder'} = \@optOrder;
 
     $next = 'unknown';
     my $i;
@@ -699,6 +755,7 @@ sub check
 	    $arg = $o->get('-cl_option');
 	    $o->set('-used' => 'yes');
 	    my $name = $o->get('-name');
+	    push @optOrder, $name;
 
 	    if ($o->get('-param') eq 'yes')
 	    {
@@ -717,7 +774,8 @@ sub check
 			    if (++$i >= @{$params{'-argv'}});
 			$l = $params{'-argv'}[$i];
 			push @{$optWithPar{$name}{'value'}},
-			@{ConfigFile::splitQuotedLine($l, "at <$arg>")};
+			@{ConfigFile::splitQuotedLine($l, "at <$arg>",
+			  $replaceEnvVar eq 'no')};
 		    }
 		    else
 		    {
@@ -727,8 +785,10 @@ sub check
 				if defined $optWithPar{$name}{'value'};
 
 			    $l = $params{'-argv'}[$i];
+			    $optWithPar{$name}{'value'} = ();
 			    push @{$optWithPar{$name}{'value'}},
-			    @{ConfigFile::splitQuotedLine($l, "at <$arg>")};
+			    @{ConfigFile::splitQuotedLine($l, "at <$arg>",
+			      $replaceEnvVar eq 'no')};
 			}
 			else
 			{
@@ -815,7 +875,11 @@ sub check
     # load and merge configuration file
     if ($configFile)
     {
-	my $cf = ConfigFile->new('-file' => $configFile);
+	my $cf = ConfigFile->new('-file' => $configFile,
+				 '-replaceEnvVar' => $replaceEnvVar,
+				 '-ignoreLeadingWhiteSpace' =>
+				 $self->{'paramList'}{'-ignoreLeadingWhiteSpace'}
+	    );
 
 	my $kp_cf_file = $cf->getKeysPointer();
 	my $kp_definition = $self->{'keysPointer'};
@@ -987,7 +1051,6 @@ sub check
 	    if $o->get('-priority') eq 'cf';
 
 	my $name = $o->get('-name');
-#	print "\t$name\n";
 
 	if (defined $optWithoutPar{$name})
 	{
@@ -997,6 +1060,10 @@ sub check
 	{
 	    delete $optWithPar{$name};
 	}
+	if ($name eq $map)     # $map = list mapping name
+	{
+	    @parList = ();     # unset param list also
+	}
     }
 
     # '-must_be' "uberpr"ufen
@@ -1012,7 +1079,7 @@ sub check
 	}
 	else # hat Parameter
 	{
-	    die "missing option <$name param>\n$Help"
+	    die "missing option with parameter <$name param>\n$Help"
 		unless ($optWithPar{$name});
 	}
     }
@@ -1108,6 +1175,29 @@ sub check
     }
 }
 
+
+########################################
+# get names of options which are set
+sub getOptNamesSet
+{
+    my $self = shift;
+
+    # Defaultwerte f"ur Parameter setzen
+    my (%params) = ('-type' => undef # 'withPar' or 'withoutPar'
+	);
+
+    &::checkObjectParams(\%params, \@_, 'CheckParam::getOptNames',
+			 ['-type']);
+
+    my $type = $params{'-type'};
+    die "type of CheckParam->getOptNames must be withPar or withoutPar"
+	unless $type =~ /\AwithPar\Z|\AwithoutPar\Z/;
+
+    my $opts = $self->{'Erg'}{$type};
+    return sort keys %$opts;
+}
+
+
 ########################################
 sub print
 {
@@ -1129,10 +1219,11 @@ sub print
     foreach $k (sort keys %$optWithPar)
     {
 	my $hidden = $self->{'namePointer'}{$k}->get('-hidden');
-	my $show = $params{'-showHidden'} || $hidden eq 'no';
+	my $show = $params{'-showHidden'} eq 'yes' || $hidden eq 'no';
+	next unless $show;
 	if (not defined $optWithPar->{$k}{'value'})
 	{
-	    print "\t\t$k\t<undef>\n" if $show;
+	    print "\t\t$k\t<undef>\n";
 	}
 	elsif (ref($optWithPar->{$k}{'value'}) eq 'ARRAY')
 	{
@@ -1140,26 +1231,26 @@ sub print
 	    my $p;
 	    foreach $p (@{$optWithPar->{$k}{'value'}})
 	    {
-		print "\t\t\t<$p>\n" if $show;
+		print "\t\t\t<$p>\n";
 	    }
 	}
 	else
 	{
-	    print "\t\t$k\t<", $optWithPar->{$k}{'value'}, ">\n" if $show;
+	    print "\t\t$k\t<", $optWithPar->{$k}{'value'}, ">\n";
 	}
     }
     print "\toptions without parameters:\n";
     foreach $k (sort keys %$optWithoutPar)
     {
 	my $hidden = $self->{'namePointer'}{$k}->get('-hidden');
-	my $show = $params{'-showHidden'} || $hidden eq 'no';
+	my $show = $params{'-showHidden'} eq 'yes' || $hidden eq 'no';
 	print "\t\t$k\n" if $show;
     }
     my $lp = '';
     $lp = ' (' . $self->{'listMapping'} . ')'
 	if defined $self->{'listMapping'};
     print "\tlist parameters$lp:\n";
-    foreach $k (sort @$parList)
+    foreach $k (@$parList)
     {
 	print "\t\t<$k>\n";
     }
@@ -1176,7 +1267,12 @@ sub getOptWithPar
     die "option <$par> does not have a parameter, use getOptWithoutPar"
 	unless $self->{'namePointer'}{$par}->get('-param') eq 'yes';
 
-    return $self->{'Erg'}{'withPar'}{$par}{'value'};
+    my $r = $self->{'Erg'}{'withPar'}{$par}{'value'};
+    if (ref($r) eq 'ARRAY')
+    {
+	$r = undef if @$r == 0;
+    }
+    return $r;
 }
 
 ########################################
@@ -1209,6 +1305,16 @@ sub getNoListPar
     return scalar(@{$self->{'Erg'}{'listPar'}});
 }
 
+########################################
+# order of options in @ARGV, if one option is used multiple times
+# it's also multiple times in the list
+sub getOptOrder
+{
+    my $self = shift;
+
+    return @{$self->{'Erg'}{'optOrder'}};
+}
+
 
 ######################################################################
 # Iterator f"ur die List-Parameter
