@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2009)
+#   Copyright (C) Heinz-Josef Claes (2001-2012)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,13 +20,162 @@
 
 
 
-push @VERSION, '$Id: fileDir.pl 354 2009-03-15 08:39:18Z hjc $ ';
+use Digest::MD5 qw(md5_hex);
+use Fcntl qw(O_RDWR O_CREAT);
+use POSIX;
 
 require 'prLog.pl';
 require 'forkProc.pl';
 
 use strict;
 
+
+# copies a file
+# returns:
+#          1: ok
+#          0: error
+############################################################
+sub copyFile
+{
+    my $source = shift;
+    my $target = shift;
+
+    local *SOURCE;
+    local *TARGET;
+    unless (sysopen(SOURCE, $source, O_RDONLY))
+    {
+	return 0;
+    }
+    unless (sysopen(TARGET, $target, O_CREAT | O_WRONLY))
+    {
+	return 0;
+    }
+    my $buffer;
+    while (sysread(SOURCE, $buffer, 10*1024**2))
+    {
+	syswrite(TARGET, $buffer);
+    }
+
+    close(TARGET);
+    close(SOURCE);
+
+    return 1;
+}
+
+############################################################
+sub copySymLink
+{
+    my $source = shift;    # symlink source
+    my $target = shift;    # symlink target
+
+    my $l = readlink $source;
+    unless ($l)
+    {
+	return 0;
+    }
+    unless (symlink $l, $target)
+    {
+	return 0;
+    }
+    else
+    {
+	return 1;
+    }
+}
+
+
+############################################################
+# use cp_a to copy a directory
+# $ignoreError = 1 -> ignore error
+# $ignoreError = 0 -> stop at error
+sub copyDir
+{
+    my ($from, $to, $tmp, $prLog, $ignoreError) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'cp',
+			   '-param' =>
+			   ['-r', "$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["copying of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["copying (with 'cp -r') of <$from> to <$to> reported:",
+		       @$out]);
+	exit 1 unless $ignoreError;
+    }
+}
+
+
+############################################################
+# use mv to move a directory
+sub mvComm
+{
+    my ($from, $to, $tmp, $prLog) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'mv',
+			   '-param' =>
+			   ["$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out],
+		      '-exit' => 1);
+    }
+}
+
+
+
+############################################################
+# &::readDirStbu($dir, __FILE__, __LINE__, $prLog);
+sub readDirStbu
+{
+    my ($dir, $file, $line, $pattern, $prLog) = (@_);
+
+    local *DIR;
+    opendir(DIR, "$dir") or
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["cannot opendir <$dir>, exiting"],
+		      '-add' => [$file, $line],
+		      '-exit' => 1);
+    my (@dirs, $entry);
+    while ($entry = readdir DIR)
+    {
+	next unless $entry =~ /$pattern/;
+	push @dirs, $entry;
+    }
+    closedir(DIR);
+
+    return (sort @dirs);
+}
+
+
 # checks if a file is a symlink and deletes it if wanted
 # return values (if not exiting):
 #               0: no symlink
@@ -38,7 +187,7 @@ sub checkDelSymLink
     my $prLog = shift;
     my $delExit = shift;    # set bits:
                             #  bit 0:  0 = do not delete
-                            #          1 = delete synlink
+                            #          1 = delete symlink
                             #              if not possible, exit
                             #  bit 1:  0 = do not exit (if exists)
                             #          1 = exit
@@ -73,16 +222,16 @@ sub splitFileDir
     my $name = shift;
 
     return (undef, undef) unless $name;
-    return ('.', $name) unless ($name =~/\//);    # nur einfacher Dateiname
+    return ('.', $name) unless ($name =~/\//);    # simple file name only
 
     my ($dir, $file) = $name =~ /^(.*)\/(.*)$/s;
-    $dir = '/' if ($dir eq '');                   # gilt, falls z.B. /filename
+    $dir = '/' if ($dir eq '');                   # if eg. /filename
     return ($dir, $file);
 }
 
 
 ############################################################
-# Parameter kann directory oder Datei sein
+# Parameter may be directory or file
 %main::absolutePathCache = ();
 @main::absolutePathCache = ();
 sub absolutePath
@@ -91,16 +240,16 @@ sub absolutePath
 
     return undef unless $dir;
 
-    # Erst im Hash nachsehen, ob das schon ermittelt wurde
+    # First look up in the hash table if that's already calculated
     return $main::absolutePathCache{$dir}
         if exists $main::absolutePathCache{$dir};
     my $dirSave = $dir;
 
-    # Falls Datei selbst ein symlink ist, solange folgen, bis aufgelst
+    # if file itself is a hard link, follow the hard link(s) to something real
     my $file = '';
 
 
-    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relativer Pfad
+    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relative path
 
     if (-e $dir)
     {
@@ -124,11 +273,11 @@ sub absolutePath
     }
 
     my $ret = (-e "$dir$file") ? "$dir$file" : undef;
-    $ret =~ s/\/\//\//g;       # // -> /
-    $ret =~ s/\/\.\//\//g;     # /./ -> /
-    $ret =~ s/\A(.+)\/\Z/$1/;  # remove trailing /
-    $ret =~ s/\A\.\///;        # ./xxx -> xxx
-    $ret =~ s/\/\.\Z//;        # xxx/. -> xxx
+    $ret =~ s/\/+/\//g;                # // -> /
+    while ($ret =~ s/\/\.\//\//g) {}   # /./ -> /
+    $ret =~ s/\A(.+?)\/+\Z/$1/;        # remove trailing /
+    $ret =~ s/\A\.\///;                # ./xxx -> xxx
+    $ret =~ s/\/\.\Z//;                # xxx/. -> xxx
 
     # eliminate /../
     my (@ret) = split(/\//, $ret);
@@ -201,7 +350,7 @@ sub relPath
 ############################################################
 sub uniqFileName
 {
-    my $prefix = shift;                 # z.B. '/tmp/test-'
+    my $prefix = shift;                 # eg. '/tmp/test-'
 
     my $suffix;
     do
@@ -215,7 +364,7 @@ sub uniqFileName
 
 
 ############################################################
-# testet, ob subDir ein Unterverzeichnis von dir ist
+# tests if subDir is a sub directory of dir
 sub isSubDir
 {
     my $dir = shift;
@@ -262,17 +411,21 @@ sub makeFilePath
     my $prLog = shift;
 
     $path =~ m#\A(.*)/.*\Z#s;
-    &makeDirPath($1);
+    &makeDirPath($1, $prLog);
 }
 
 
 ############################################################
 # like `mkdir -p`, all permissions set to 0700
+# success: returns 1
+# no success: returns 0
 sub makeDirPath
 {
     my $path = shift;
     my $prLog = shift;
 
+    return unless $path;
+
     my @p;
     return if -e $path;
 
@@ -280,12 +433,14 @@ sub makeDirPath
 
     &makeDirPath($p1, $prLog);
 
-    $prLog->print('-kind' => 'E',
-		  '-str' => ["cannot create directory <$path>"],
-		  '-exit' => 1)
-	unless mkdir $path, 0700;
-
-    return;
+    unless (mkdir $path, 0700)
+    {
+	return if -e $path;
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot create directory <$path>"]);
+	return 0;
+    }
+    return 1;
 }
 
 
@@ -335,7 +490,7 @@ sub checkLockFile
     local *FILE;
     if (-f $lockFile)
     {
-	open(FILE, "< $lockFile") or
+	open(FILE, '<', $lockFile) or
 	    $prLog->print('-kind' => 'E',
 			  '-str' => ["cannot read lock file <$lockFile>"],
 			  '-exit' => 1);
@@ -344,7 +499,7 @@ sub checkLockFile
 	close(FILE);
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["strange format in lock file <$lockFile>, " .
-				 "line is <$pid>\n"],
+				 "line is <$pid>"],
 		      '-exit' => 1)
 	    unless ($pid =~ /\A\d+\Z/o);
 	if (kill(0, $pid) == 1)   # alte Instanz luft noch
@@ -366,7 +521,7 @@ sub checkLockFile
 		  '-str' => ["creating lock file <$lockFile>"]);
 
     &::checkDelSymLink($lockFile, $prLog, 0x01);
-    open(FILE, "> $lockFile") or
+    open(FILE, '>', $lockFile) or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot create lock file <$lockFile>"],
 		      '-exit' => 1);
@@ -375,6 +530,47 @@ sub checkLockFile
 }
 
 
+############################################################
+sub calcMD5
+{
+    my $filename = shift;
+    my $prLog = shift;
+
+    local *FROM;
+    unless (sysopen(FROM, $filename, O_RDONLY))
+    {
+	return undef;
+    }
+    my $md5 = Digest::MD5->new();
+    my $buffer;
+    while (sysread(FROM, $buffer, 1024*1024))
+    {
+	$md5->add($buffer);
+    }
+    close(FROM);
+    return $md5->hexdigest();
+}
+
+
+############################################################
+sub readSymLinkCalcMd5
+{
+    my $filename = shift;
+    my $prLog = shift;
+
+    my $l = readlink $filename;
+    unless ($l)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot read symlink value <$filename>"]);
+	return undef;
+    }
+    my $md5 = Digest::MD5->new();
+    $md5->add($l);
+    return $md5->hexdigest();
+}
+
+
 ############################################################
 # Objekt kann zum (wiederholten) Abfragen von Informationen
 # ber eine Datei verwendet werden.
@@ -588,8 +784,9 @@ sub readDir
     my $includeDirs = $self->{'includeDirs'};
     my $ignoreReadError = $self->{'ignoreReadError'};
 
-    if ($self->{'printDepth'} eq 'yes' and
-	$self->{'printedDepth'} != $depth)
+
+    if ($self->{'printDepth'} eq 'yes' and defined $depth
+	and $self->{'printedDepth'} != $depth)
     {
 	$self->{'printedDepth'} = $depth;
 	$prLog->print('-kind' => $self->{'printDepthPrlogKind'},
@@ -597,8 +794,6 @@ sub readDir
     }
 
     return unless $dir;
-#    $dir = &::absolutePath($dir);
-#    print "------------dir = <$dir> ----------\n";
     if (@$includeDirs)
     {
 	my $ignore = 1;
@@ -655,19 +850,10 @@ sub readDir
 	return if $ignore;
     }
 
-
     local *DIR;
     unless (opendir(DIR, $dir))
     {
-	if ($ignoreReadError eq 'no')
-	{
-	    $prLog->print('-kind' => $prLogErr,
-			  '-str' =>
-			  ["cannot opendir <$dir>, exiting",
-			   "check your source and clean your backup!"],
-			  '-exit' => $exit);
-	}
-	elsif ($ignoreReadError eq 'onlyPrintMessage')
+	if ($ignoreReadError)
 	{
 	    $prLog->print('-kind' => $prLogErr,
 			  '-str' => ["cannot opendir <$dir>"]);
@@ -675,7 +861,11 @@ sub readDir
 	}
 	else
 	{
-	    return;
+	    $prLog->print('-kind' => $prLogErr,
+			  '-str' =>
+			  ["cannot opendir <$dir>, exiting",
+			   "check your source and clean your backup!"],
+			  '-exit' => $exit);
 	}
     }
 
@@ -713,8 +903,17 @@ sub readDir
 	}
 	unless (-r $entry)
 	{
-	    $prLog->print('-kind' => $prLogWarn,
-			  '-str' => ["no permissions to read <$entry>"]);
+	    if ($ignoreReadError)
+	    {
+		$prLog->print('-kind' => $prLogWarn,
+			      '-str' => ["no permissions to read <$entry>"]);
+	    }
+	    else
+	    {
+		$prLog->print('-kind' => $prLogErr,
+			      '-str' => ["no permissions to read <$entry>"],
+			      '-exit' => 1);
+	    }
 	    next;
 	}
 	if (-d $entry)       # Dieses Directory mu beim Kopieren
