@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2009)
+#   Copyright (C) Heinz-Josef Claes (2001-2012)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,8 +20,6 @@
 
 
 
-push @VERSION, '$Id: forkProc.pl 354 2009-03-15 08:39:18Z hjc $ ';
-
 use Carp;
 use strict;
 
@@ -33,6 +31,65 @@ require 'prLog.pl';
 require 'fileDir.pl';
 
 
+############################################################
+# returns (), if all programs were found, list of
+# missing programs instead
+sub checkProgExists
+{
+    my ($prLog, @progs) = (@_);
+
+    my $prog;
+    my (@fault) = ();
+    foreach $prog ('which', @progs)
+    {
+	my $f = forkProc->new('-exec' => 'which',
+			      '-param' => [$prog],
+			      '-outRandom' => '/tmp/which-',
+			      '-prLog' => $prLog);
+	$f->wait();
+	if ($f->get('-what' => 'status'))
+	{
+	    push @fault, $prog;
+	    return @fault if $prog eq 'which';
+	}
+    }
+    return @fault;
+}
+
+
+############################################################
+sub waitForFile
+{
+    my $existingFile = shift;
+
+    my @intervall = (.01, .01, .02, .02, .04, .05, .05,
+		      .05, .05, .05, .05, .05, .05, .05, .05,
+		     .05, .05, .05, .05, .05, .05, .05, .05); # 1 second
+
+    if (not -e $existingFile)
+    {
+	my $i;
+	foreach $i (@intervall)
+	{
+	    select(undef, undef, undef, $i);
+	    if (-e $existingFile)
+	    {
+		select(undef, undef, undef, $intervall[0]);
+		last;
+	    }
+	}
+
+	if ($i == 0)
+	{
+	    system "/bin/sync";
+	    return not -e $existingFile;
+	}
+    }
+
+    return 0;    # found file
+}
+
+
 ############################################################
 package simpleFork;
 
@@ -123,6 +180,452 @@ sub signal
 }
 
 
+############################################################
+# replacement for open(FILE, "| prog")
+# pipe into another program
+# must call method close at the end
+package pipeToFork;
+
+sub new
+{
+    my $class = shift;
+    my $self = {};
+
+    my (%params) = ('-exec'        => undef,
+		    '-param'       => [],
+		    '-workingDir'  => undef,
+
+		    '-stdout'      => undef, # hat Vorrang vor '-outRandom',
+		                             # falls gesetzt
+		    '-stderr'      => undef, # hat Vorrang vor '-outRandom',
+		                             # falls gesetzt
+		    '-outRandom'   => undef, # wenn auf String gesetzt,
+		                             # (z.B. "/tmp/test") wird
+		                             # dieser als Anfang des
+		                             # Dateinamens fr '-stdout'
+		                             # und '-stderr' genommen
+		    '-delStdout'   => 'yes', # lsche Datei fr stdout
+		                             # automatisch
+
+		    '-prLog'       => undef,
+
+		    '-info'        => undef, # beliebige zustzliche Info
+
+		    '-prLogError'  => 'E',
+		    '-exitIfError' => 1      # Exit-Wert bei Error
+		    );
+
+    &::checkObjectParams(\%params, \@_, 'pipeToFork::new',
+			 ['-exec', '-prLog']);
+    &::setParamsDirect($self, \%params);
+
+    local *PARENT;
+    my $fd = *PARENT;
+    my $proc = $params{'-exec'};
+    my $par = $params{'-param'};     # Pointer auf Parameter
+    my $dir = $params{'-workingDir'};
+    my $stdout = $params{'-stdout'};
+    my $stderr = $params{'-stderr'};
+    my $outRandom = $params{'-outRandom'};
+    my $prLog = $params{'-prLog'};
+    my $err = $params{'-prLogError'};
+    my $ex = $params{'-exitIfError'};
+
+    # zufllig Dateinamen erstellen, falls ntig
+    if ($outRandom ne undef)
+    {
+	$self->{'stdout'} = $stdout = &::uniqFileName($outRandom)
+	    if $stdout eq undef;
+	$self->{'stderr'} = $stderr = &::uniqFileName($outRandom)
+	    if $stderr eq undef;
+    }
+
+    my $child;
+    unless (pipe $child, $fd)
+    {
+	$prLog->print('-kind' => $err,
+		      '-str' => ["cannot open pipe for <$proc>"]);
+	exit $ex;
+    }
+
+    my $pid = fork;
+
+    unless ($pid)      # in the child
+    {
+	close PARENT;
+	unless (open(STDIN, "<&=" . fileno($child)))
+	{
+	    $prLog->print('-kind' => $err,
+			  '-str' => ["cannot pipe STDIN for <$proc>"]);
+	    exit $ex;
+	}
+	if (defined $dir)
+	{
+	    unless (chdir "$dir")
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot chdir to <$dir> for <$proc>"]);
+		exit $ex;
+	    }
+	}
+	if (defined $stdout)
+	{
+	    unless (open STDOUT, ">", $stdout)
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot write <$stdout> when starting <$proc>"]);
+		exit $ex;
+	    }
+	    chown 0600, $stdout;
+	}
+	if (defined $stderr)
+	{
+	    unless (open STDERR, ">", $stderr)
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot dup stderr when starting $proc"]);
+		exit $ex;
+	    }
+	    chown 0600, $stderr;
+	}
+
+	$proc =~ s/(\s)/\\$1/g;
+	unless (exec $proc, @$par)
+	{
+	    $prLog->print('-kind' => $err,
+			  '-str' => ["cannot exec $proc @$par"]);
+	    exit $ex;
+	}
+    }
+
+    close $child;              # in the parent
+    $self->{'fd'} = $fd;
+    $self->{'pid'} = $pid;
+    $self->{'status'} = undef;
+
+    bless $self, $class;
+}
+
+
+##################################################
+sub print
+{
+    my ($self, @buffer) = @_;
+
+    my ($i);
+    for ($i = 0 ; $i < @buffer ; $i++)
+    {
+	syswrite $self->{'fd'}, $buffer[$i];
+    }
+}
+
+
+##################################################
+sub get
+{
+    my $self = shift;
+
+    my (%params) = ('-what'        => undef);
+
+    &::checkObjectParams(\%params, \@_, 'forkProc::get',
+			 ['-what']);
+
+    return undef unless defined $self->{$params{'-what'}};
+    return $self->{$params{'-what'}};
+}
+
+
+##################################################
+sub wait
+{
+    my $self = shift;
+
+    if ($self->{'fd'})
+    {
+	close $self->{'fd'};
+	$self->{'fd'} = undef;
+
+	waitpid $self->{'pid'}, 0;
+#	waitpid $self->{'pid'}, WNOHANG;
+	$self->{'status'} = $? >> 8 if $self->{'status'} eq undef;
+    }
+}
+
+
+##################################################
+sub getSTDOUT
+{
+    my $self = shift;
+
+    local *FILE;
+    my ($l, @lines);
+    open(FILE, "< " . $self->{'stdout'})
+	or return [];
+    while ($l = <FILE>)
+    {
+	chomp $l;
+	push @lines, $l;
+    }
+    close(FILE);
+    return \@lines;
+}
+
+
+##################################################
+sub getSTDERR
+{
+    my $self = shift;
+
+    local *FILE;
+    my ($l, @lines);
+    open(FILE, "< " . $self->{'stderr'})
+	or return [];
+    while ($l = <FILE>)
+    {
+	chomp $l;
+	push @lines, $l;
+    }
+    close(FILE);
+    return \@lines;
+}
+
+
+##################################################
+sub delSTDOUT
+{
+    my $self = shift;
+
+    unlink $self->{'stdout'} if ($self->{'stdout'} and
+				 $self->{'stdout'} ne '/dev/null');
+    $self->{'stdout'} = undef;
+}
+
+
+##################################################
+sub close
+{
+    my $self = shift;
+
+    $self->wait();
+
+    unlink $self->{'stderr'} if ($self->{'stderr'} and
+				 $self->{'stderr'} ne '/dev/null');
+
+    $self->delSTDOUT() if ($self->{'delStdout'} eq 'yes');
+}
+
+
+############################################################
+# replacement for open(FILE, "prog |")
+# pipes from another program into this one
+# must call method close at the end
+# ATTENTION: if prLog write to STDOUT, messages are also piped!
+package pipeFromFork;
+
+sub new
+{
+    my $class = shift;
+    my $self = {};
+
+    my (%params) = ('-exec'        => undef,
+		    '-param'       => [],
+		    '-workingDir'  => undef,
+
+		    '-stdin'      => undef,
+
+		    '-stderr'      => undef, # hat Vorrang vor '-outRandom',
+		                             # falls gesetzt
+		    '-outRandom'   => undef, # wenn auf String gesetzt,
+		                             # (z.B. "/tmp/test") wird
+		                             # dieser als Anfang des
+		                             # Dateinamens fr
+		                             # '-stderr' genommen
+
+		    '-prLog'       => undef,
+
+		    '-info'        => undef, # beliebige zustzliche Info
+
+		    '-prLogError'  => 'E',
+		    '-exitIfError' => 1      # Exit-Wert bei Error
+		    );
+
+    &::checkObjectParams(\%params, \@_, 'pipeFromFork::new',
+			 ['-exec', '-prLog']);
+    &::setParamsDirect($self, \%params);
+
+    local *PARENT;
+    my $fd = *PARENT;
+    my $proc = $params{'-exec'};
+    my $par = $params{'-param'};     # Pointer auf Parameter
+    my $dir = $params{'-workingDir'};
+    my $stdin = $params{'-stdin'};
+    my $stderr = $params{'-stderr'};
+    my $outRandom = $params{'-outRandom'};
+    my $prLog = $params{'-prLog'};
+    my $err = $params{'-prLogError'};
+    my $ex = $params{'-exitIfError'};
+
+    # zufllig Dateinamen erstellen, falls ntig
+    if ($outRandom ne undef)
+    {
+	$self->{'stderr'} = $stderr = &::uniqFileName($outRandom)
+	    if $stderr eq undef;
+    }
+
+    my $child;
+    unless (pipe $fd, $child)
+    {
+	$prLog->print('-kind' => $err,
+		      '-str' => ["cannot open pipe for <$proc>"]);
+	exit $ex;
+    }
+
+    my $pid = fork;
+
+    unless ($pid)      # in the child
+    {
+	close PARENT;
+	unless (open(STDOUT, ">&=" . fileno($child)))
+	{
+	    $prLog->print('-kind' => $err,
+			  '-str' => ["cannot pipe STDOUT for <$proc>"]);
+	    exit $ex;
+	}
+	if (defined $dir)
+	{
+	    unless (chdir "$dir")
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot chdir to <$dir> for <$proc>"]);
+		exit $ex;
+	    }
+	}
+	if (defined $stdin)
+	{
+	    unless (sysopen(STDIN, $stdin, 0))
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot read <$stdin> when starting <$proc>"]);
+		exit $ex;
+	    }
+	}
+	if (defined $stderr)
+	{
+	    unless (open STDERR, ">", $stderr)
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' =>
+			      ["Cannot dup stderr when starting $proc"]);
+		exit $ex;
+	    }
+	    chown 0600, $stderr;
+	}
+
+	$proc =~ s/(\s)/\\$1/g;
+	unless (exec $proc, @$par)
+	{
+	    $prLog->print('-kind' => $err,
+			  '-str' => ["cannot exec $proc @$par"]);
+	    exit $ex;
+	}
+    }
+
+    close $child;              # in the parent
+    $self->{'fd'} = $fd;
+    $self->{'pid'} = $pid;
+    $self->{'status'} = undef;
+    $self->{'lineNr'} = 0;
+
+    bless $self, $class;
+}
+
+
+##################################################
+sub sysread
+{
+    my ($self, $buffer, $blockSize) = @_;
+
+    return sysread $self->{'fd'}, $$buffer, $blockSize;
+}
+
+##################################################
+sub read
+{
+    my $self = shift;
+
+    ++$self->{'lineNr'};
+    local *IN = $self->{'fd'};
+    return <IN>;
+}
+
+
+##################################################
+sub get
+{
+    my $self = shift;
+
+    my (%params) = ('-what'        => undef);
+
+    &::checkObjectParams(\%params, \@_, 'forkProc::get',
+			 ['-what']);
+
+    return undef unless defined $self->{$params{'-what'}};
+    return $self->{$params{'-what'}};
+}
+
+
+##################################################
+sub wait
+{
+    my $self = shift;
+
+    if ($self->{'fd'})
+    {
+	close $self->{'fd'};
+	$self->{'fd'} = undef;
+
+	waitpid $self->{'pid'}, 0;
+	$self->{'status'} = $? >> 8 if $self->{'status'} eq undef;
+    }
+}
+
+
+##################################################
+sub getSTDERR
+{
+    my $self = shift;
+
+    local *FILE;
+    my ($l, @lines);
+    open(FILE, "< " . $self->{'stderr'})
+	or return [];
+    while ($l = <FILE>)
+    {
+	chomp $l;
+	push @lines, $l;
+    }
+    close(FILE);
+    return \@lines;
+}
+
+
+##################################################
+sub close
+{
+    my $self = shift;
+
+    $self->wait();
+
+    unlink $self->{'stderr'} if ($self->{'stderr'} and
+				 $self->{'stderr'} ne '/dev/null');
+}
+
+
 ############################################################
 package forkProc;
 
@@ -228,7 +731,6 @@ sub new
 	    chown 0600, $stderr;
 	}
 
-	$proc =~ s/(\s)/\\$1/g;
 	unless (exec $proc, @$par)
 	{
 	    $prLog->print('-kind' => $err,
@@ -722,12 +1224,12 @@ sub wait
             if $self->{'waitTime'} < $self->{'maxWaitTime'};
     }
 
-    if ($self->{debug} and
-       $self->{waitTime} > 0)
+    if ($self->{'debug'} >= 4 and
+       $self->{'waitTime'} > 0)
     {
-        $self->{prLog}->print(-kind => "D",
+        $self->{'prLog'}->print('-kind' => "D",
                               -str => ["Scheduler: waiting ".
-                                       $self->{waitTime}]);
+                                       $self->{'waitTime'}]);
     }
 
     select(undef, undef, undef, $self->{'waitTime'});
@@ -846,4 +1348,5 @@ sub get
     return @{$self->{'queue'}} > 0 ? shift @{$self->{'queue'}} : undef;
 }
 
+
 1
