@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2000-2008)
+#   Copyright (C) Heinz-Josef Claes (2000-2012)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -19,13 +19,13 @@
 #
 
 
-push @VERSION, '$Id: prLog.pl 350 2008-12-13 15:50:19Z hjc $ ';
-
 
 use IO::Handle;
 use strict;
 
 require 'checkObjPar.pl';
+require 'fileDir.pl';
+
 
 ############################################################
 package printLog;
@@ -43,7 +43,7 @@ sub new
 		    '-withTime'       => 'yes',
 		    '-withPID'        => 'yes',
 		    '-hostname'       => '',
-		    '-maxFilelen'     => 1e6,
+		    '-maxFilelen'     => 1e6, # 0 means unlimited file length
 		    '-noOfOldFiles'   => 5,
 		    '-filter'         => {},  # Replace in Output Key by Value
 		    '-closeFile'      => 'no',# ffnet und schliet die Datei
@@ -51,7 +51,7 @@ sub new
 		    '-multiprint'     => 'no',# pos an's Ende vor jedem print,
 		                              # mehrere Appl. print in ein log
 		    '-saveLogs'       => 'no',# nicht rundschreiben, wenn 'yes'
-		    '-compressWith'   => 'undef' # Komprimierungsprogr.,
+		    '-compressWith'   => undef   # Komprimierungsprogr.,
 		                                 # z.B. 'gzip -9', 'bzip2'
 		                                 # nur zusammen mit '-saveLogs'
 		    );
@@ -60,6 +60,9 @@ sub new
 
     $self->{'param'} = \%params;    # Parameter an Objekt binden
 
+    $self->{'fifo'} = undef;       # not forked
+    $self->{'pidNumber'} = $$;     # don't use pid of forked process
+
     my (%kindhash, $k, $maxlen);
     $maxlen = 0;
     foreach $k (@{$self->{'param'}{'-kind'}})    # Art der m"oglichen
@@ -122,6 +125,55 @@ sub new
 }
 
 
+##################################################
+sub fork
+{
+    my $self = shift;
+    my $pathTo_stbuLogPl = shift;
+
+    return if defined $self->{'fifo'};
+
+    my $fifo = ::uniqFileName('/tmp/prLog-');
+    POSIX::mkfifo($fifo, 0600) or
+	die "cannot mknod <$fifo> for printLog";
+    $self->{'fifo'} = $fifo;
+
+# build options for call of stbuLog.pl
+    my (@opts) = ('--readFile' => $fifo);
+
+    if (defined $self->{'param'}{'-file'})
+    {
+	push @opts, ('--out' => $self->{'param'}{'-file'},
+		     '--maxFilelen' => $self->{'param'}{'-maxFilelen'});
+
+	if ($self->{'param'}{'-saveLogs'} eq 'yes')
+	{
+	    push @opts, ('--saveLogs');
+	    push @opts, ('--compressWith' => $self->{'param'}{'-compressWith'})
+		if (defined $self->{'param'}{'-compressWith'});
+	}
+	else  # write round
+	{
+	    push @opts, ('--noOldFiles' => $self->{'param'}{'-noOfOldFiles'});
+	}
+    }
+
+    my $p = printLog->new();
+#print "$$ STARTING stbuLog.pl with @opts\n";
+    my $logD = forkProc->new('-exec' => "$pathTo_stbuLogPl/stbuLog.pl",
+			     '-param' => \@opts,
+			     '-prLog' => $p);
+    die "cannot start <$pathTo_stbuLogPl/stbuLog.pl>"
+	unless ($logD->processRuns());
+
+    $self->{'logD'} = $logD;
+    local *FIFO;
+    open(FIFO, '>', $fifo) or
+	die "cannot open <$fifo> for writing";
+    $self->{'fifoFD'} = *FIFO;
+}
+
+
 ##################################################
 sub setStopAtNoMessages
 {
@@ -194,11 +246,13 @@ sub print
 
     # Defaultwerte f"ur Parameter setzen
     my (%params) = ('-kind' => undef,
-		    '-str' => [],          # Liste mit auszugebenden Strings
+		    '-str' => [],  # Liste mit auszugebenden Strings
+		    '-add' => [],  # add as comma sep. list in [] to last line
 		    '-exit' => undef
 		    );
 
-    &::checkObjectParams(\%params, \@_, 'printLog::print', ['-kind']);
+    &::checkObjectParams(\%params, \@_, 'printLog::print',
+			 ['-kind', '-str']);
 
     my $closeFile = $params{'-closeFile'};
     if ($closeFile eq 'yes')
@@ -216,7 +270,8 @@ sub print
     $k = $self->{'kindhash'}{'?'} unless ($k);
 
     my $t = $self->__getTime();
-    my $pid = $self->{'param'}{'-withPID'} eq 'yes' ? sprintf("%5d ", $$) : '';
+    my $pid = $self->{'param'}{'-withPID'} eq 'yes' ?
+	sprintf("%5d ", $self->{'pidNumber'}) : '';
 
     my $hostname = $self->{'param'}{'-hostname'};
     $hostname = "$hostname " if length($hostname);
@@ -231,6 +286,12 @@ sub print
         }
         push @a, "$k $t" . $hostname . "$pid$a";
     }
+    my $add = $params{'-add'};
+    if (@$add)
+    {
+	$a[@a-1] .= ' [' . join(', ', @$add) . ']';
+    }
+
     $self->__print(@a);
 
     if (defined($self->{'encounteredStop'}->{$kind}) and
@@ -303,6 +364,7 @@ sub __print         # interne Methode
     local *FILE = $self->{'filehandle'};
 
     if ($self->{'filesize'} >= $self->{'param'}{'-maxFilelen'}
+	and $self->{'param'}{'-maxFilelen'} != 0
 	and $self->{'param'}{'-file'})
     {
 	if ($self->{'param'}{'-saveLogs'} eq 'yes')   # Mit Datum wegsichern
@@ -312,7 +374,7 @@ sub __print         # interne Methode
 	    my (@t) = (localtime(time))[5,4,3,2,1,0];
 	    $t[0] += 1900;              # localtime liefert Zeit seit 1900
 	    $t[1]++;                    # Monat fngt bei 1 an
-	    my $t = sprintf("%04d.%02d.%02d_%02d:%02d:%02d", @t);
+	    my $t = sprintf("%04d.%02d.%02d_%02d.%02d.%02d", @t);
 	    my $t0 = $t;
 	    $t =~ s/\s/_/;         # blank zwischen Datum und Uhrzeit ersetzen
 	    $t =~ s/\s$//;         # blank am Ende lschen
@@ -387,60 +449,92 @@ Continue:;
     $self->__reallyPrint(\@_);
 }
 
+
 ##################################################
 sub __reallyPrint
 {
     my $self = shift;
     my $lines = shift;
-    local *FILE = $self->{'filehandle'};
 
-    if ($self->{'param'}{'-multiprint'} eq 'yes')
+#print "in __reallyPrint, fifo = ", $self->{'fifo'}, " <@$lines>\n";
+    if ($self->{'fifo'})
     {
-	my (@s) = stat($self->{'param'}{'-file'});
-	if (@s > 0)              # file still exists
+	local *FIFO;
+	*FIFO = $self->{'fifoFD'};
+	foreach my $l (@$lines)
 	{
-	    seek(FILE, 0, 2);
-	}
-	else        # file has been moved from another process
-	{
-	    close(FILE);
-	    open(FILE, ">>" . $self->{'param'}{'-file'}) or
-		die "cannot write to file <", $self->{'param'}{'-file'}, ">\n";
-	    $self->{'filehandle'} = *FILE;
+#print "PRINT TO FIFO: <$l>\n";
+	    print FIFO $l, "\n";
 	}
+	FIFO->autoflush(1);
     }
-
-    my ($l);
-    foreach $l (@$lines)
+    else
     {
-	my ($p) = "$l\n";
-	$self->{'filesize'} += length($p);
+	local *FILE = $self->{'filehandle'};
 
-	print FILE $p or
-	    die "cannot write to file <", $self->{'param'}{'-file'}, ">\n";
+	if ($self->{'param'}{'-multiprint'} eq 'yes')
+	{
+	    my (@s) = stat($self->{'param'}{'-file'});
+	    if (@s > 0)              # file still exists
+	    {
+		seek(FILE, 0, 2);
+	    }
+	    else        # file has been moved from another process
+	    {
+		close(FILE);
+		open(FILE, ">>" . $self->{'param'}{'-file'}) or
+		    die "cannot write to file <", $self->{'param'}{'-file'}, ">\n";
+		$self->{'filehandle'} = *FILE;
+	    }
+	}
+
+	my ($l);
+	foreach $l (@$lines)
+	{
+	    my ($p) = "$l\n";
+	    $self->{'filesize'} += length($p);
+
+	    print FILE $p or
+		die "cannot write to file <", $self->{'param'}{'-file'}, ">";
+	}
+	FILE->autoflush(1);
     }
-    FILE->autoflush(1);
 }
 
 
 ##################################################
 sub DESTROY
 {
-    my ($self) = shift;
+    my $self = shift;
 
-    local *FILE = $self->{'filehandle'};
-    if (*FILE ne *STDOUT)
+#print "0\n";
+#print "fifo=",$self->{'fifo'},"\n";
+    if (defined $self->{'fifo'})
     {
-	close(FILE) or
-	    die "cannot close <", $self->{'param'}{'-file'}, ">\n";
+	local *FIFO;
+	*FIFO = $self->{'fifoFD'};
+	print FIFO "__FINISH__\n";
+	close(FIFO);
+#print "1\n";
+	$self->{'logD'}->wait();
+#print "2\n";
+	unlink $self->{'fifo'};
+    }
+    else
+    {
+	local *FILE = $self->{'filehandle'};
+	if (*FILE ne *STDOUT)
+	{
+	    close(FILE) or
+		die "cannot close <", $self->{'param'}{'-file'}, ">\n";
+	}
+	# wait sets $? if there are no child processes. This causes the
+	# entire program to exit with return code 255 - really undesired.
+	# So we have to circumvent this. Unfortunately setting $? = 0
+	# causes the program to *always* exit with return code zero. So it
+	# seems better to not wait() at all - what is it good for anyway?
+	#wait;      # wait for execed compression
     }
-
-    # wait sets $? if there are no child processes. This causes the
-    # entire program to exit with return code 255 - really undesired.
-    # So we have to circumvent this. Unfortunately setting $? = 0
-    # causes the program to *always* exit with return code zero. So it
-    # seems better to not wait() at all - what is it good for anyway?
-    #wait;      # wait for execed compression
 }
 
 
@@ -458,9 +552,28 @@ sub new
     &::checkObjectParams(\%params, \@_, 'printLogMultiple::new', []);
     &::setParamsDirect($self, \%params);
 
+    $self->{'pathTo_stbuLogPl'} = undef;
+
     bless $self, $class;
 }
 
+
+##################################################
+sub fork
+{
+    my $self = shift;
+    my $pathTo_stbuLogPl = shift;
+
+    $self->{'pathTo_stbuLogPl'} = $pathTo_stbuLogPl;
+
+    my $prLog;
+    foreach $prLog (@{$self->{'prLogs'}})
+    {
+	$prLog->fork($pathTo_stbuLogPl);
+    }
+}
+
+
 ##################################################
 sub encountered
 {
@@ -476,6 +589,21 @@ sub encountered
 }
 
 
+##################################################
+sub addEncounter
+{
+    my $self = shift;
+
+    # Defaultwerte f"ur Parameter setzen
+    my (%params) = ('-kind' => undef,
+		    '-add' => 0);
+
+    &::checkObjectParams(\%params, \@_, 'printLog::addEncounter', ['-kind']);
+
+    $self->{'encountered'}->{$params{'-kind'}} += $params{'-add'};
+}
+
+
 ##################################################
 sub add      # weitere prLog hinzufgen
 {
@@ -487,6 +615,9 @@ sub add      # weitere prLog hinzufgen
 			 ['-prLogs']);
 
     push @{$self->{'prLogs'}}, @{$params{'-prLogs'}};
+
+    $self->fork($self->{'pathTo_stbuLogPl'})
+	if defined $self->{'pathTo_stbuLogPl'};
 }
 
 
@@ -551,3 +682,5 @@ sub pr
 	$prLog->pr(@_);
     }
 }
+
+1
