@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2002-2008)
+#   Copyright (C) Heinz-Josef Claes (2002-2012)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,10 +19,10 @@
 #
 
 
-push @VERSION, '$Id: version.pl 343 2008-08-10 11:12:24Z hjc $ ';
+$main::STOREBACKUPVERSION = "3.3.1";
 
 
-sub printVersions
+sub printVersion
 {
     my ($ARGV, $par) = @_;
 
@@ -39,22 +39,7 @@ sub printVersions
     }
     return if ($flag == 0);
 
-    my ($sum);
-
-    foreach $entry (sort @VERSION)
-    {
-	print "$entry\n";
-	$entry =~ /^\$Id:\s+(\S+)\s+(\S+)/;
-
-	my ($letter, $s);
-	foreach $letter (split(//, $1))
-	{
-	    $s += (ord($letter) - ord('0') + 1) / (ord('z') - ord('a'));
-	}
-	$sum += ($s / length($1)) * $2;
-    }
-    $sum /= 2;
-    printf "overall checksum = %.4f\n",$sum / @VERSION;
+    print "version $main::STOREBACKUPVERSION\n";
 
     exit 0;
 }
